=== modified file 'bzrlib/cleanup.py'
--- bzrlib/cleanup.py	2009-09-23 04:21:10 +0000
+++ bzrlib/cleanup.py	2009-09-23 06:00:51 +0000
@@ -42,8 +42,8 @@
     finally:
         run_cleanups([cleanup_func_a, cleanup_func_b], ...)
 
-Developers can use the `-Dcleanup` debug flag to always propagate errors from
-cleanup functions.
+Developers can use the `-Dcleanup` debug flag to cause cleanup errors to be
+reported in the UI as well as logged.
 
 XXX: what about the case where do_something succeeds, but cleanup fails, and
 that matters?
@@ -63,9 +63,11 @@
     trace,
     )
 
-def _log_cleanup_error():
+def _log_cleanup_error(exc):
     trace.mutter('Cleanup failed:')
     trace.log_exception_quietly()
+    if 'cleanup' in debug.debug_flags:
+        trace.warning('bzr: warning: Cleanup failed: %s', exc)
 
 
 def run_cleanup(func, *args, **kwargs):
@@ -78,26 +80,23 @@
         func(*args, **kwargs)
     except KeyboardInterrupt:
         raise
-    except:
-        _log_cleanup_error()
-        if 'cleanup' in debug.debug_flags:
-            raise
-        return False
-    return True
-
-
-def run_cleanup_reporting_errors(func, *args, **kwargs):
-    try:
-        func(*args, **kwargs)
-    except KeyboardInterrupt:
-        raise
-    except Exception, e:
-        _log_cleanup_error()
-        trace.warning('Cleanup function %r failed: %s', func, e)
-        if 'cleanup' in debug.debug_flags:
-            raise
-        return False
-    return True
+    except Exception, exc:
+        _log_cleanup_error(exc)
+        return False
+    return True
+
+
+#def run_cleanup_reporting_errors(func, *args, **kwargs):
+#    try:
+#        func(*args, **kwargs)
+#    except KeyboardInterrupt:
+#        raise
+#    except Exception, exc:
+#        trace.mutter('Cleanup failed:')
+#        trace.log_exception_quietly()
+#        trace.warning('Cleanup failed: %s', exc)
+#        return False
+#    return True
 
 
 def run_cleanups(funcs, on_error='log'):

=== modified file 'bzrlib/tests/test_cleanup.py'
--- bzrlib/tests/test_cleanup.py	2009-09-23 04:30:21 +0000
+++ bzrlib/tests/test_cleanup.py	2009-09-23 06:00:51 +0000
@@ -14,13 +14,18 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
+from cStringIO import StringIO
 import re
 
-from bzrlib.tests import TestCase
 from bzrlib.cleanup import (
     do_with_cleanups,
     run_cleanup,
     )
+from bzrlib.tests import TestCase
+from bzrlib import (
+    debug,
+    trace,
+    )
 
 
 class CleanupsTestCase(TestCase):
@@ -71,6 +76,17 @@
         self.assertFalse(run_cleanup(self.failing_cleanup))
         self.assertLogContains('Cleanup failed:.*failing_cleanup goes boom')
 
+    def test_cleanup_error_debug_flag(self):
+        """The -Dcleanup debug flag causes cleanup errors to be propagated."""
+        log = StringIO()
+        trace.push_log_file(log)
+        debug.debug_flags.add('cleanup')
+        self.assertFalse(run_cleanup(self.failing_cleanup))
+        self.assertContainsRe(
+            log.getvalue(),
+            "bzr: warning: Cleanup failed:.*failing_cleanup goes boom")
+        
+
     def test_prior_error_cleanup_succeeds(self):
         """Calling run_cleanup from a finally block will not interfere with an
         exception from the try block.

